@@ -26,15 +26,12 @@
 #include <errno.h>
 #include <signal.h>
 #include <time.h>
+#include <fcntl.h>
+#include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#include <netinet/ip.h>
-#include <netinet/udp.h>
-#include <netinet/tcp.h>
-#include <netinet/ip_icmp.h>
-#include <netinet/igmp.h>
 #if defined(RAIN_IPV6)
   #include <netinet/ip6.h>
   #include <netinet/icmp6.h>
@@ -46,7 +43,6 @@
 #include "rain_command_line.h"
 #include "rain_tcp.h"
 #include "rain_udp.h"
-#include "rain_icmp.h"
 #if defined(RAIN_IPV6)
   #include "rain_icmp6.h"
 #endif
@@ -55,10 +51,25 @@
 #include "rain_resolv.h"
 #include "rain_signal.h"
 #include "rain_string.h"
+#include "rain_mem.h"
+#include "rain_defaults.h"
+#include "rain_services.h"
+#include "rain_icmp.h"
+#include "rain_errno.h"
 
-#define RAIN_VERSION "1.2.4"
+#define RAIN_VERSION "1.2.6"
 
 #define TRUE  (1)
 #define FALSE (0)
 
+#define UID_CHECK(uid,opt) { \
+  switch(uid) { \
+    case  0: { break; } \
+    default:{ \
+      fprintf(stderr,"\n- - Error: uid %d is not permitted to use %s (must be root)\n",uid,opt); \
+      exit(1); \
+    } \
+  } \
+}
+
 #endif /* _RAIN_H */
